@@ -1,7 +1,10 @@
-from typing import Any, Dict, List, Type, TypeVar, cast
+from typing import Any, Dict, List, Type, TypeVar
 
 import attr
 
+from ..models.dataset_info_response_200_allowed_teams_item import (
+    DatasetInfoResponse200AllowedTeamsItem,
+)
 from ..models.dataset_info_response_200_data_source import (
     DatasetInfoResponse200DataSource,
 )
@@ -20,7 +23,7 @@ class DatasetInfoResponse200:
     data_source: DatasetInfoResponse200DataSource
     data_store: DatasetInfoResponse200DataStore
     owning_organization: str
-    allowed_teams: List[Any]
+    allowed_teams: List[DatasetInfoResponse200AllowedTeamsItem]
     is_active: int
     is_public: int
     description: str
@@ -35,7 +38,7 @@ class DatasetInfoResponse200:
     is_unreported: int
     is_foreign: int
     jobs_enabled: int
-    tags: List[str]
+    tags: List[Any]
     additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)
 
     def to_dict(self) -> Dict[str, Any]:
@@ -47,7 +50,7 @@ class DatasetInfoResponse200:
         owning_organization = self.owning_organization
         allowed_teams = []
         for allowed_teams_item_data in self.allowed_teams:
-            allowed_teams_item = allowed_teams_item_data
+            allowed_teams_item = allowed_teams_item_data.to_dict()
 
             allowed_teams.append(allowed_teams_item)
 
@@ -65,7 +68,11 @@ class DatasetInfoResponse200:
         is_unreported = self.is_unreported
         is_foreign = self.is_foreign
         jobs_enabled = self.jobs_enabled
-        tags = self.tags
+        tags = []
+        for tags_item_data in self.tags:
+            tags_item = tags_item_data
+
+            tags.append(tags_item)
 
         field_dict: Dict[str, Any] = {}
         field_dict.update(self.additional_properties)
@@ -110,7 +117,9 @@ class DatasetInfoResponse200:
         allowed_teams = []
         _allowed_teams = d.pop("allowedTeams")
         for allowed_teams_item_data in _allowed_teams:
-            allowed_teams_item = allowed_teams_item_data
+            allowed_teams_item = DatasetInfoResponse200AllowedTeamsItem.from_dict(
+                allowed_teams_item_data
+            )
 
             allowed_teams.append(allowed_teams_item)
 
@@ -142,7 +151,12 @@ class DatasetInfoResponse200:
 
         jobs_enabled = d.pop("jobsEnabled")
 
-        tags = cast(List[str], d.pop("tags"))
+        tags = []
+        _tags = d.pop("tags")
+        for tags_item_data in _tags:
+            tags_item = tags_item_data
+
+            tags.append(tags_item)
 
         dataset_info_response_200 = cls(
             name=name,
