@@ -1,4 +1,5 @@
 import os
+import warnings
 from functools import lru_cache
 from pathlib import Path
 from tempfile import TemporaryDirectory
@@ -17,10 +18,6 @@ from webknossos.client._generated.api.default import (
     datastore_list,
     new_dataset_name_is_valid,
 )
-from webknossos.client._generated.models import (
-    DatasetFinishUploadJsonBody,
-    DatasetReserveUploadJsonBody,
-)
 from webknossos.client._resumable import Resumable
 from webknossos.client.context import _get_context, _WebknossosContext
 from webknossos.dataset import Dataset
@@ -33,7 +30,7 @@ class LayerToLink(NamedTuple):
     dataset_name: str
     layer_name: str
     new_layer_name: Optional[str] = None
-    organization_name: Optional[
+    organization_id: Optional[
         str
     ] = None  # defaults to the user's organization before uploading
 
@@ -43,7 +40,7 @@ class LayerToLink(NamedTuple):
             "dataSetName": self.dataset_name,
             "layerName": self.layer_name,
             "newLayerName": self.new_layer_name,
-            "organizationName": self.organization_name or context.organization,
+            "organizationName": self.organization_id or context.organization_id,
         }
 
 
@@ -77,6 +74,11 @@ def upload_dataset(
     layers_to_link: Optional[List[LayerToLink]] = None,
     jobs: Optional[int] = None,
 ) -> str:
+    from webknossos.client._generated.models import (
+        DatasetFinishUploadJsonBody,
+        DatasetReserveUploadJsonBody,
+    )
+
     if new_dataset_name is None:
         new_dataset_name = dataset.name
     if layers_to_link is None:
@@ -84,6 +86,10 @@ def upload_dataset(
     context = _get_context()
     layer_names_to_link = set(i.new_layer_name or i.layer_name for i in layers_to_link)
     if len(layer_names_to_link.intersection(dataset.layers.keys())) > 0:
+        warnings.warn(
+            "Excluding the following layers from upload, since they will be linked: "
+            + f"{layer_names_to_link.intersection(dataset.layers.keys())}"
+        )
         with TemporaryDirectory() as tmpdir:
             tmp_ds = dataset.shallow_copy_dataset(
                 tmpdir, name=dataset.name, layers_to_ignore=layer_names_to_link
@@ -107,13 +113,13 @@ def upload_dataset(
     if "PYTEST_CURRENT_TEST" in os.environ:
         simultaneous_uploads = 1
     response = new_dataset_name_is_valid.sync_detailed(
-        organization_name=context.organization,
+        organization_name=context.organization_id,
         data_set_name=new_dataset_name,
         client=context.generated_auth_client,
     )
     assert (
         response.status_code == 200
-    ), f"Dataset name {context.organization}/{new_dataset_name} does not seem to be valid: {response}"
+    ), f"Dataset name {context.organization_id}/{new_dataset_name} does not seem to be valid: {response}"
     for _ in range(MAXIMUM_RETRY_COUNT):
         response = dataset_reserve_upload.sync_detailed(
             client=datastore_client,
@@ -121,7 +127,7 @@ def upload_dataset(
             json_body=DatasetReserveUploadJsonBody.from_dict(
                 {
                     "uploadId": upload_id,
-                    "organization": context.organization,
+                    "organization": context.organization_id,
                     "name": new_dataset_name,
                     "totalFileCount": len(file_infos),
                     "initialTeams": [],
@@ -138,7 +144,7 @@ def upload_dataset(
             f"{datastore_url}/data/datasets?token={datastore_token}",
             simultaneous_uploads=simultaneous_uploads,
             query={
-                "owningOrganization": context.organization,
+                "owningOrganization": context.organization_id,
                 "name": new_dataset_name,
                 "totalFileCount": len(file_infos),
             },
@@ -161,7 +167,7 @@ def upload_dataset(
             json_body=DatasetFinishUploadJsonBody.from_dict(
                 {
                     "uploadId": upload_id,
-                    "organization": context.organization,
+                    "organization": context.organization_id,
                     "name": new_dataset_name,
                     "needsConversion": False,
                     "layersToLink": [layer.as_json() for layer in layers_to_link],
@@ -173,4 +179,4 @@ def upload_dataset(
     else:
         assert response.status_code == 200, response
 
-    return f"{context.url}/datasets/{context.organization}/{dataset.name}/view"
+    return f"{context.url}/datasets/{context.organization_id}/{new_dataset_name}/view"
