@@ -1,12 +1,11 @@
 import copy
 import json
-import os
-import shutil
+import re
 import warnings
 from argparse import Namespace
 from contextlib import nullcontext
-from os import PathLike, makedirs
-from os.path import basename, join, normpath
+from os import PathLike
+from os.path import relpath
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
@@ -23,23 +22,31 @@ from typing import (
 
 import attr
 import numpy as np
-import wkw
 from boltons.typeutils import make_sentinel
 
+from ..geometry.vec3_int import Vec3IntLike
+from ._array import ArrayException, ArrayInfo, BaseArray, DataFormat
+
 if TYPE_CHECKING:
-    from webknossos.client._upload_dataset import LayerToLink
+    from ..client._upload_dataset import LayerToLink
 
-from webknossos.dataset._utils.infer_bounding_box_existing_files import (
-    infer_bounding_box_existing_files,
+from ..geometry import BoundingBox, Mag
+from ..utils import (
+    copy_directory_with_symlinks,
+    copytree,
+    get_executor_for_args,
+    is_fs_path,
+    is_symlink,
+    make_upath,
+    rmtree,
 )
-from webknossos.geometry import BoundingBox, Mag
-from webknossos.utils import copy_directory_with_symlinks, get_executor_for_args
-
+from ._utils.infer_bounding_box_existing_files import infer_bounding_box_existing_files
 from .layer import (
     Layer,
     SegmentationLayer,
     _dtype_per_channel_to_element_class,
     _dtype_per_layer_to_dtype_per_channel,
+    _get_sharding_parameters,
     _normalize_dtype_per_channel,
     _normalize_dtype_per_layer,
 )
@@ -56,9 +63,15 @@ from .properties import (
 from .view import View
 
 DEFAULT_BIT_DEPTH = 8
-
+DEFAULT_DATA_FORMAT = DataFormat.WKW
 PROPERTIES_FILE_NAME = "datasource-properties.json"
 
+_DATASET_URL_REGEX = re.compile(
+    r"^(?P<webknossos_url>https?://.*)/datasets/"
+    + r"(?P<organization_id>[^/]*)/(?P<dataset_name>[^/]*)(/(view)?)?"
+    + r"(\?token=(?P<sharing_token>[^#]*))?"
+)
+
 
 def _copy_job(args: Tuple[View, View, int]) -> None:
     (source_view, target_view, _) = args
@@ -66,6 +79,17 @@ def _copy_job(args: Tuple[View, View, int]) -> None:
     target_view.write(source_view.read())
 
 
+def _find_array_info(layer_path: Path) -> Optional[ArrayInfo]:
+    for f in layer_path.iterdir():
+        if f.is_dir():
+            try:
+                array = BaseArray.open(f)
+                return array.info
+            except ArrayException:
+                pass
+    return None
+
+
 _UNSET = make_sentinel("UNSET", var_name="_UNSET")
 
 _UNSPECIFIED_SCALE_FROM_OPEN = make_sentinel(
@@ -100,17 +124,20 @@ class Dataset:
         Currently exist_ok=True is the deprecated default and will change in future releases.
         Please use `Dataset.open` if you intend to open an existing dataset and don't want/need the creation behavior.
         """
-        dataset_path = Path(dataset_path)
+        dataset_path = make_upath(dataset_path)
 
         dataset_existed_already = (
-            dataset_path.is_dir() and next(dataset_path.iterdir(), None) is not None
+            dataset_path.exists()
+            and dataset_path.is_dir()
+            and next(dataset_path.iterdir(), None) is not None
         )
 
         if dataset_existed_already:
             if exist_ok == _UNSET:
                 warnings.warn(
                     f"[DEPRECATION] You are creating/opening a dataset at a non-empty folder {dataset_path} without setting exist_ok=True. "
-                    + "This will fail in future releases, please supply exist_ok=True explicitly then."
+                    + "This will fail in future releases, please supply exist_ok=True explicitly then.",
+                    DeprecationWarning,
                 )
                 exist_ok = True
             if not exist_ok:
@@ -126,7 +153,7 @@ class Dataset:
             ), f"Creation of Dataset at {dataset_path} failed, because a file already exists at this path."
             # Create directories on disk and write datasource-properties.json
             try:
-                makedirs(dataset_path, exist_ok=True)
+                dataset_path.mkdir(parents=True, exist_ok=True)
             except OSError as e:
                 raise type(e)(
                     "Creation of Dataset {} failed. ".format(dataset_path) + repr(e)
@@ -136,48 +163,43 @@ class Dataset:
             assert (
                 scale is not None
             ), "When creating a new dataset, the scale must be given, e.g. as Dataset(path, scale=(10, 10, 16.8))"
-            name = name or basename(normpath(dataset_path))
+            name = name or dataset_path.absolute().name
             dataset_properties = DatasetProperties(
                 id={"name": name, "team": ""}, scale=scale, data_layers=[]
             )
-            with open(
-                dataset_path / PROPERTIES_FILE_NAME, "w", encoding="utf-8"
+            with (dataset_path / PROPERTIES_FILE_NAME).open(
+                "w", encoding="utf-8"
             ) as outfile:
                 json.dump(
                     dataset_converter.unstructure(dataset_properties), outfile, indent=4
                 )
 
         self.path = dataset_path
-        with open(
-            self.path / PROPERTIES_FILE_NAME, encoding="utf-8"
-        ) as datasource_properties:
-            data = json.load(datasource_properties)
+        self._properties: DatasetProperties = self._load_properties()
+        self._last_read_properties = copy.deepcopy(self._properties)
 
-            self._properties: DatasetProperties = dataset_converter.structure(
-                data, DatasetProperties
+        self._layers: Dict[str, Layer] = {}
+        # construct self.layer
+        for layer_properties in self._properties.data_layers:
+            num_channels = _extract_num_channels(
+                layer_properties.num_channels,
+                make_upath(dataset_path),
+                layer_properties.name,
+                layer_properties.mags[0].mag
+                if len(layer_properties.mags) > 0
+                else None,
             )
+            layer_properties.num_channels = num_channels
 
-            self._layers: Dict[str, Layer] = {}
-            # construct self.layer
-            for layer_properties in self._properties.data_layers:
-                num_channels = _extract_num_channels(
-                    layer_properties.num_channels,
-                    Path(dataset_path),
-                    layer_properties.name,
-                    layer_properties.wkw_resolutions[0].resolution
-                    if len(layer_properties.wkw_resolutions) > 0
-                    else None,
-                )
-                layer_properties.num_channels = num_channels
-
-                layer = self._initialize_layer_from_properties(layer_properties)
-                self._layers[layer_properties.name] = layer
+            layer = self._initialize_layer_from_properties(layer_properties)
+            self._layers[layer_properties.name] = layer
 
         if dataset_existed_already:
             if scale is None:
                 warnings.warn(
                     "[DEPRECATION] Please always supply the scale when using the constructor Dataset(your_path, scale=your_scale)."
-                    + "If you just want to open an existing dataset, please use Dataset.open(your_path)."
+                    + "If you just want to open an existing dataset, please use Dataset.open(your_path).",
+                    DeprecationWarning,
                 )
             elif scale == _UNSPECIFIED_SCALE_FROM_OPEN:
                 pass
@@ -200,7 +222,7 @@ class Dataset:
 
         The `dataset_path` refers to the top level directory of the dataset (excluding layer or magnification names).
         """
-        dataset_path = Path(dataset_path)
+        dataset_path = make_upath(dataset_path)
         assert (
             dataset_path.exists()
         ), f"Cannot open Dataset: Couldn't find {dataset_path}"
@@ -218,18 +240,56 @@ class Dataset:
     @classmethod
     def download(
         cls,
-        dataset_name: str,
-        organization_name: Optional[str] = None,
+        dataset_name_or_url: str,
+        organization_id: Optional[str] = None,
+        sharing_token: Optional[str] = None,
+        webknossos_url: Optional[str] = None,
         bbox: Optional[BoundingBox] = None,
-        layers: Optional[List[str]] = None,
+        layers: Union[List[str], str, None] = None,
         mags: Optional[List[Mag]] = None,
         path: Optional[Union[PathLike, str]] = None,
         exist_ok: bool = False,
-        webknossos_url: Optional[str] = None,
     ) -> "Dataset":
+        """Downloads a dataset and returns the Dataset instance.
+
+        * `dataset_name_or_url` may be a dataset name or a full URL to a dataset view, e.g.
+          `https://webknossos.org/datasets/scalable_minds/l4_sample_dev/view`
+          If a URL is used, `organization_id`, `webknossos_url` and `sharing_token` must not be set.
+        * `organization_id` may be supplied if a dataset name was used in the previous argument,
+          it defaults to your current organization from the `webknossos_context`.
+          You can find your `organization_id` [here](https://webknossos.org/auth/token).
+        * `sharing_token` may be supplied if a dataset name was used and can specify a sharing token.
+        * `webknossos_url` may be supplied if a dataset name was used,
+          and allows to specifiy in which webknossos instance to search for the dataset.
+          It defaults to the url from your current `webknossos_context`, using https://webknossos.org as a fallback.
+        * `bbox`, `layers`, and `mags` specify which parts of the dataset to download.
+          If nothing is specified the whole image, all layers, and all mags are downloaded respectively.
+        * `path` and `exist_ok` specify where to save the downloaded dataset and whether to overwrite
+          if the `path` exists.
+
+        The `webknossos_url` specifies in which webknossos instance to search for the dataset. By default the
+        configured url from `webknossos_context` is used, using https://webknossos.org as a fallback.
+        """
+
         from webknossos.client._download_dataset import download_dataset
         from webknossos.client.context import _get_context, webknossos_context
 
+        match = re.match(_DATASET_URL_REGEX, dataset_name_or_url)
+        if match is not None:
+            assert (
+                organization_id is None
+                and sharing_token is None
+                and webknossos_url is None
+            ), (
+                "When Dataset.download() is be called with an annotation url, "
+                + "e.g. Dataset.download('https://webknossos.org/datasets/scalable_minds/l4_sample_dev/view'), "
+                + "organization_id, sharing_token and webknossos_url must not be set."
+            )
+            dataset_name_or_url = match.group("dataset_name")
+            organization_id = match.group("organization_id")
+            sharing_token = match.group("sharing_token")
+            webknossos_url = match.group("webknossos_url")
+
         if webknossos_url is not None and webknossos_url != _get_context().url:
             warnings.warn(
                 f"The supplied url {webknossos_url} does not match your current context {_get_context().url}. "
@@ -242,9 +302,19 @@ class Dataset:
         else:
             context = nullcontext()
 
+        if isinstance(layers, str):
+            layers = [layers]
+
         with context:
             return download_dataset(
-                dataset_name, organization_name, bbox, layers, mags, path, exist_ok
+                dataset_name_or_url,
+                organization_id=organization_id,
+                sharing_token=sharing_token,
+                bbox=bbox,
+                layers=layers,
+                mags=mags,
+                path=path,
+                exist_ok=exist_ok,
             )
 
     @property
@@ -296,6 +366,7 @@ class Dataset:
         dtype_per_layer: Optional[Union[str, np.dtype, type]] = None,
         dtype_per_channel: Optional[Union[str, np.dtype, type]] = None,
         num_channels: Optional[int] = None,
+        data_format: Union[str, DataFormat] = DEFAULT_DATA_FORMAT,
         **kwargs: Any,
     ) -> Layer:
         """
@@ -307,6 +378,8 @@ class Dataset:
 
         Creates the folder `layer_name` in the directory of `self.path`.
 
+        WKW layers can only be added to datasets on local file systems.
+
         The return type is `webknossos.dataset.layer.Layer`.
 
         This function raises an `IndexError` if the specified `layer_name` already exists.
@@ -346,6 +419,10 @@ class Dataset:
                 )
             )
 
+        assert (
+            is_fs_path(self.path) or data_format != DataFormat.WKW
+        ), "Cannot create WKW layers in remote datasets. Use `data_format='zarr'`."
+
         layer_properties = LayerProperties(
             name=layer_name,
             category=category,
@@ -353,9 +430,9 @@ class Dataset:
             element_class=_dtype_per_channel_to_element_class(
                 dtype_per_channel, num_channels
             ),
-            wkw_resolutions=[],
+            mags=[],
             num_channels=num_channels,
-            data_format="wkw",
+            data_format=DataFormat(data_format),
         )
 
         if category == COLOR_CATEGORY:
@@ -395,6 +472,7 @@ class Dataset:
         dtype_per_layer: Optional[Union[str, np.dtype, type]] = None,
         dtype_per_channel: Optional[Union[str, np.dtype, type]] = None,
         num_channels: Optional[int] = None,
+        data_format: Union[str, DataFormat] = DEFAULT_DATA_FORMAT,
         **kwargs: Any,
     ) -> Layer:
         """
@@ -453,6 +531,7 @@ class Dataset:
                 dtype_per_layer=dtype_per_layer,
                 dtype_per_channel=dtype_per_channel,
                 num_channels=num_channels,
+                data_format=DataFormat(data_format),
                 **kwargs,
             )
 
@@ -463,7 +542,7 @@ class Dataset:
             )
 
         layer_properties = copy.copy(other_layer._properties)
-        layer_properties.wkw_resolutions = []
+        layer_properties.mags = []
         layer_properties.name = layer_name
 
         self._properties.data_layers += [layer_properties]
@@ -479,21 +558,23 @@ class Dataset:
         return self._layers[layer_name]
 
     def add_layer_for_existing_files(
-        self, layer_name: str, category: LayerCategoryType, **kwargs: Any
+        self,
+        layer_name: str,
+        category: LayerCategoryType,
+        **kwargs: Any,
     ) -> Layer:
         assert layer_name not in self.layers, f"Layer {layer_name} already exists!"
-        mag_headers = list((self.path / layer_name).glob("*/header.wkw"))
 
+        array_info = _find_array_info(self.path / layer_name)
         assert (
-            len(mag_headers) != 0
-        ), f"Could not find any header.wkw files in {self.path / layer_name}, cannot add layer."
-        with wkw.Dataset.open(str(mag_headers[0].parent)) as wkw_dataset:
-            header = wkw_dataset.header
+            array_info is not None
+        ), f"Could not find any valid mags in {self.path /layer_name}. Cannot add layer."
         layer = self.add_layer(
             layer_name,
             category=category,
-            num_channels=header.num_channels,
-            dtype_per_channel=header.voxel_type,
+            num_channels=array_info.num_channels,
+            dtype_per_channel=array_info.voxel_type,
+            data_format=array_info.data_format,
             **kwargs,
         )
         for mag_dir in layer.path.iterdir():
@@ -504,14 +585,15 @@ class Dataset:
 
     def get_segmentation_layer(self) -> SegmentationLayer:
         """
-        Returns the only segmentation layer. Prefer `get_segmentation_layers`,
-        because multiple segmentation layers can exist.
+        Deprecated, please use `get_segmentation_layers()`.
 
+        Returns the only segmentation layer.
         Fails with a IndexError if there are multiple segmentation layers or none.
         """
 
         warnings.warn(
-            "[DEPRECATION] get_segmentation_layer() fails if no or more than one segmentation layer exists. Prefer get_segmentation_layers()."
+            "[DEPRECATION] get_segmentation_layer() fails if no or more than one segmentation layer exists. Prefer get_segmentation_layers().",
+            DeprecationWarning,
         )
         return cast(
             SegmentationLayer,
@@ -530,20 +612,21 @@ class Dataset:
 
     def get_color_layer(self) -> Layer:
         """
-        Returns the only color layer. Prefer `get_color_layers`, because multiple
-        color layers can exist.
+        Deprecated, please use `get_color_layers()`.
 
+        Returns the only color layer.
         Fails with a RuntimeError if there are multiple color layers or none.
         """
+        warnings.warn(
+            "[DEPRECATION] get_color_layer() fails if no or more than one color layer exists. Prefer get_color_layers().",
+            DeprecationWarning,
+        )
         return self._get_layer_by_category(COLOR_CATEGORY)
 
     def get_color_layers(self) -> List[Layer]:
         """
         Returns all color layers.
         """
-        warnings.warn(
-            "[DEPRECATION] get_color_layer() fails if no or more than one color layer exists. Prefer get_color_layers()."
-        )
         return [
             cast(Layer, layer)
             for layer in self.layers.values()
@@ -565,11 +648,11 @@ class Dataset:
             layer for layer in self._properties.data_layers if layer.name != layer_name
         ]
         # delete files on disk
-        if layer_path.is_symlink():
+        if is_symlink(layer_path):
             layer_path.unlink()
         else:
             # rmtree does not recurse into linked dirs, but removes the link
-            shutil.rmtree(layer_path)
+            rmtree(layer_path)
         self._export_as_json()
 
     def add_symlink_layer(
@@ -585,12 +668,13 @@ class Dataset:
         (or vice versa).
         If make_relative is True, the symlink is made relative to the current dataset path.
         If new_layer_name is None, the name of the foreign layer is used.
+        Symlinked layers can only be added to datasets on local file systems.
         """
 
         if isinstance(foreign_layer, Layer):
             foreign_layer_path = foreign_layer.path
         else:
-            foreign_layer_path = Path(foreign_layer)
+            foreign_layer_path = make_upath(foreign_layer)
 
         foreign_layer_name = foreign_layer_path.name
         layer_name = (
@@ -601,12 +685,20 @@ class Dataset:
                 f"Cannot create symlink to {foreign_layer_path}. This dataset already has a layer called {layer_name}."
             )
 
+        assert is_fs_path(
+            self.path
+        ), f"Cannot create symlinks in remote dataset {self.path}"
+        assert is_fs_path(
+            foreign_layer_path
+        ), f"Cannot create symlink to remote layer {foreign_layer_path}"
+
         foreign_layer_symlink_path = (
-            Path(os.path.relpath(foreign_layer_path, self.path))
+            Path(relpath(foreign_layer_path, self.path))
             if make_relative
             else foreign_layer_path.resolve()
         )
-        os.symlink(foreign_layer_symlink_path, join(self.path, layer_name))
+
+        (self.path / layer_name).symlink_to(foreign_layer_symlink_path)
         original_layer = Dataset.open(foreign_layer_path.parent).get_layer(
             foreign_layer_name
         )
@@ -634,7 +726,7 @@ class Dataset:
         if isinstance(foreign_layer, Layer):
             foreign_layer_path = foreign_layer.path
         else:
-            foreign_layer_path = Path(foreign_layer)
+            foreign_layer_path = make_upath(foreign_layer)
 
         foreign_layer_name = foreign_layer_path.name
         layer_name = (
@@ -645,7 +737,7 @@ class Dataset:
                 f"Cannot copy {foreign_layer_path}. This dataset already has a layer called {layer_name}."
             )
 
-        shutil.copytree(foreign_layer_path, join(self.path, layer_name))
+        copytree(foreign_layer_path, self.path / layer_name)
         original_layer = Dataset.open(foreign_layer_path.parent).get_layer(
             foreign_layer_name
         )
@@ -663,17 +755,42 @@ class Dataset:
         self,
         new_dataset_path: Union[str, Path],
         scale: Optional[Tuple[float, float, float]] = None,
-        block_len: Optional[int] = None,
-        file_len: Optional[int] = None,
+        chunk_size: Optional[Union[Vec3IntLike, int]] = None,
+        chunks_per_shard: Optional[Union[Vec3IntLike, int]] = None,
+        data_format: Optional[Union[str, DataFormat]] = None,
         compress: Optional[bool] = None,
+        block_len: Optional[int] = None,  # deprecated
+        file_len: Optional[int] = None,  # deprecated
         args: Optional[Namespace] = None,
     ) -> "Dataset":
         """
         Creates a new dataset at `new_dataset_path` and copies the data from the current dataset to `empty_target_ds`.
-        If not specified otherwise, the `scale`, `block_len`, `file_len` and `block_type` of the current dataset are also used for the new dataset.
+        If not specified otherwise, the `scale`, `chunk_size`, `chunks_per_shard` and `compress` of the current dataset
+        are also used for the new dataset. The method also accepts the parameters `block_len` and `file_size`,
+        which were deprecated by `chunk_size` and `chunks_per_shard`.
+        WKW layers can only be copied to datasets on local file systems.
         """
 
-        new_dataset_path = Path(new_dataset_path)
+        chunk_size, chunks_per_shard = _get_sharding_parameters(
+            chunk_size=chunk_size,
+            chunks_per_shard=chunks_per_shard,
+            block_len=block_len,
+            file_len=file_len,
+        )
+
+        new_dataset_path = make_upath(new_dataset_path)
+
+        if data_format == DataFormat.WKW:
+            assert is_fs_path(
+                new_dataset_path
+            ), "Cannot create WKW-based remote datasets. Use `data_format='zarr'` instead."
+        if data_format is None and any(
+            layer.data_format == DataFormat.WKW for layer in self.layers.values()
+        ):
+            assert is_fs_path(
+                new_dataset_path
+            ), "Cannot create WKW layers in remote datasets. Use explicit `data_format='zarr'`."
+
         if scale is None:
             scale = self.scale
         new_ds = Dataset(new_dataset_path, scale=scale, exist_ok=False)
@@ -683,9 +800,11 @@ class Dataset:
                 new_ds_properties = copy.deepcopy(
                     self.get_layer(layer_name)._properties
                 )
-                # Initializing a layer with non-empty wkw_resolutions requires that the files on disk already exist.
+                # Initializing a layer with non-empty mags requires that the files on disk already exist.
                 # The MagViews are added manually afterwards
-                new_ds_properties.wkw_resolutions = []
+                new_ds_properties.mags = []
+                if data_format is not None:
+                    new_ds_properties.data_format = DataFormat(data_format)
                 new_ds._properties.data_layers += [new_ds_properties]
                 target_layer = new_ds._initialize_layer_from_properties(
                     new_ds_properties
@@ -695,13 +814,13 @@ class Dataset:
                 bbox = self.get_layer(layer_name).bounding_box
 
                 for mag, mag_view in layer.mags.items():
-                    block_len = block_len or mag_view.header.block_len
-                    compress = compress or (
-                        mag_view.header.block_type != wkw.Header.BLOCK_TYPE_RAW
+                    chunk_size = chunk_size or mag_view.info.chunk_size
+                    compression_mode = compress or mag_view.info.compression_mode
+                    chunks_per_shard = (
+                        chunks_per_shard or mag_view.info.chunks_per_shard
                     )
-                    file_len = file_len or mag_view.header.file_len
                     target_mag = target_layer.add_mag(
-                        mag, block_len, file_len, compress
+                        mag, chunk_size, chunks_per_shard, compression_mode
                     )
 
                     target_layer.bounding_box = bbox
@@ -729,6 +848,7 @@ class Dataset:
         In addition, link all other directories in all layer directories
         to make this method robust against additional files e.g. layer/mappings/agglomerate_view.hdf5.
         This method becomes useful when exposing a dataset to webknossos.
+        Only datasets on local filesystems can be shallow copied.
         """
         new_dataset = Dataset(
             new_dataset_path, scale=self.scale, name=name or self.name, exist_ok=False
@@ -803,7 +923,8 @@ class Dataset:
         **Deprecated**, please use the constructor `Dataset()` instead.
         """
         warnings.warn(
-            "[DEPRECATION] Dataset.create() is deprecated in favor of the normal constructor Dataset()."
+            "[DEPRECATION] Dataset.create() is deprecated in favor of the normal constructor Dataset().",
+            DeprecationWarning,
         )
         return cls(dataset_path, scale, name, exist_ok=False)
 
@@ -818,21 +939,41 @@ class Dataset:
         **Deprecated**, please use the constructor `Dataset()` instead.
         """
         warnings.warn(
-            "[DEPRECATION] Dataset.get_or_create() is deprecated in favor of the normal constructor Dataset(…, exist_ok=True)."
+            "[DEPRECATION] Dataset.get_or_create() is deprecated in favor of the normal constructor Dataset(…, exist_ok=True).",
+            DeprecationWarning,
         )
         return cls(dataset_path, scale, name, exist_ok=True)
 
     def __repr__(self) -> str:
         return repr("Dataset(%s)" % self.path)
 
+    def _load_properties(self) -> DatasetProperties:
+        with (self.path / PROPERTIES_FILE_NAME).open(
+            encoding="utf-8"
+        ) as datasource_properties:
+            data = json.load(datasource_properties)
+        return dataset_converter.structure(data, DatasetProperties)
+
     def _export_as_json(self) -> None:
-        with open(self.path / PROPERTIES_FILE_NAME, "w", encoding="utf-8") as outfile:
+
+        properties_on_disk = self._load_properties()
+
+        if properties_on_disk != self._last_read_properties:
+            warnings.warn(
+                "[WARNING] While exporting the dataset's properties, properties were found on disk which are "
+                + "newer than the ones that were seen last time. The properties will be overwritten. This is "
+                + "likely happening because multiple processes changed the metadata of this dataset."
+            )
+
+        with (self.path / PROPERTIES_FILE_NAME).open("w", encoding="utf-8") as outfile:
             json.dump(
                 dataset_converter.unstructure(self._properties),
                 outfile,
                 indent=4,
             )
 
+            self._last_read_properties = copy.deepcopy(self._properties)
+
     def _initialize_layer_from_properties(self, properties: LayerProperties) -> Layer:
         if properties.category == COLOR_CATEGORY:
             return Layer(self, properties)
